using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PhoenixDLL;
using PhoenixDLL.GUI;
using System.Data.OleDb;
using PhoenixDLL.Class;
using PhoenixDLL.Class.Managers;
using PhoenixDLL.GUI.Seekers;
using PhoenixDLL.GUI.Managers;
using PhoenixCards.GUI;
using Components.SplashBar;
using System.IO;
using System.Drawing.Imaging;
using Utils.Misc;
using PhoenixDLL.Class.Exceptions;
using Utils.Logs;
using Utils.DataBase;
using PhoenixCards.Class;
using PhoenixMoney.GUI;
using System.Threading;
using PhoenixPics;
using Utils.ImageProcessing;

namespace PhoenixCards.GUI
{
    /// <summary>
    /// Fentre principale de la gestion des fiches mdicale
    /// </summary>
    public partial class fr_Main : Form
    {
        #region Variables
        private const string vFrameTxr = "Phoenix - Pata";
        /// <summary>
        /// Vrifie si on  sauver les dernires modifications
        /// </summary>
        private bool vIsSaved = true;
        /// <summary>
        /// Sql manager
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
        /// <summary>
        /// Information sur le patient courrant
        /// </summary>
        private PatientManager vPatientManager;
        /// <summary>
        /// Fichier d'vOptions
        /// </summary>
        private Options opt = Options.GetInstance();
        /// <summary>
        /// Collection des onglets 
        /// </summary>
        private List<TabPage> vTabPage = new List<TabPage>();
        /// <summary>
        /// Sauvegarde pour revenir  l'tat prcdent
        /// </summary>
        private string[] vPreviousState = new string[8];
        /// <summary>
        /// Bool pour viter que le premier remplissage ne soit considr comme une modification
        /// </summary>
        private bool vIsLoading = true;
        /// <summary>
        /// Manager of the payments
        /// </summary>
        private TimeCardManager vTimeCardManager;
        /// <summary>
        /// Contains the meetings to display in the Calendar
        /// </summary>
        private MeetingSet vMeetingSet;
        #endregion

        #region Constructeurs
        /// <summary>
        /// Constructeur
        /// </summary>
        public fr_Main()
        {
            InitializeComponent();
            SetToolStrips();
            opt.OptionChanged += new OnOptionChangedHandler(Event_OptionChanged);

            foreach (TabPage tp in tc_General.TabPages)
                vTabPage.Add(tp);

            SetTabs();

            #region Visibilit des menus
            tsMenu_Modification.Enabled = false;
            tsMenu_Supprimer.Enabled = false;
            btn_AddPrescription.Enabled = false;
            tsb_SendMail.Visible = false;

            ts_Menu.Visible = Options.Instance.PataMenu.SeekCancelVisible;
            tsMenu_SeekCancel.Checked = Options.Instance.PataMenu.PageStyleVisible;
            tsMenu_Options.Visible = Options.Instance.OptionsMenuVisible;
            tssl_HasRdv.BackColor = SystemColors.Control;
            tssl_HasRdv.Text = "";
            tssl_Debt.Text = "";
            tssddb_MissedMeeting.Visible = false;
            #endregion

            #region Pictures GUI
            tlp_GeneralPics.Dock = DockStyle.None;
            tlp_GeneralPics.Location = new Point(9999, 9999);
            lbl_WaitMessage.Location = new Point(0, 0);
            lbl_WaitMessage.Dock = DockStyle.Fill;
            lbl_WaitMessage.Text = "Slctionnez un patient!";
            #endregion
        }
        #endregion

        #region Event
        /// <summary>
        /// Manage the today's meeting of the current patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ManageToday(object sender, EventArgs e)
        {
            if (vTimeCardManager != null)
            {
                fr_Today fr = new fr_Today();
                fr.TimeCardManager = this.vTimeCardManager;
                fr.MeetingSet = vMeetingSet;
                fr.ShowDialog();
                this.UpdatePatientData();
                //vMeetingSet = fr.MeetingSet;
            }
        }
        /// <summary>
        /// Clique sur le menu de recherche
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SeekPatient(object sender, EventArgs e)
        {
            #region Ask to close the current session
            if (this.vPatientManager != null && this.vPatientManager.IDPatient > 0)
            {
                DialogResult ddr = MessageBox.Show("Faire une recherche fermera la fiche de " + vPatientManager.ToString() + "."
                    + "Voulez-vous sauvez avant de faire la recherche?",
                    "Question", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
                if (ddr == DialogResult.Cancel) return;
                else if (ddr == DialogResult.Yes) this.SaveCards();
            }
            fr_PatientSeeker fr = new fr_PatientSeeker();
            DialogResult dr = fr.ShowDialog();
            #endregion

            #region A patient was selected
            if (dr == DialogResult.OK && fr.SelectedItem.IDPatient > 0)//&& fr.PatientInfo.IsFilled)
            {
                SetFrameOnPatientSelection(fr.SelectedItem, fr.CardsInfo);
            }
            #endregion
        }
        /// <summary>
        /// Deep patient seek
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DeepSeek(object sender, EventArgs e)
        {
            if (this.vPatientManager != null && this.vPatientManager.IDPatient > 0)
            {
                DialogResult ddr = MessageBox.Show("Faire une recherche fermera la fiche de " + vPatientManager.ToString() + "."
                    + "Voulez-vous sauvez avant de faire la recherche?",
                    "Question", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question);
                if (ddr == DialogResult.Cancel) return;
                else if (ddr == DialogResult.Yes) this.SaveCards();
            }
            fr_DeepPatientSeeker fr = new fr_DeepPatientSeeker();
            DialogResult dr = fr.ShowDialog();
            if (dr == DialogResult.OK && fr.SelectedItem.IDPatient > 0)//&& fr.PatientInfo.IsFilled)
            {
                this.SetFrameOnPatientSelection(fr.SelectedItem, fr.CardsInfo);
            }
        }
        /// <summary>
        /// Clique sur le menu Options
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Options(object sender, EventArgs e)
        {
            fr_Options fr = new fr_Options();
            fr.ShowDialog();
        }
        /// <summary>
        /// Menu de demande de retour  l'tat prcdent
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Previous(object sender, EventArgs e)
        {
            #region Demande de confirmation
            DialogResult dr = MessageBox.Show("Voulez-vous revenir  la version "
                + "prcdente des fiches mdicales? Si vous cliquez sur oui toutes"
                + " les modifications seront perdues!", "Question",
                MessageBoxButtons.YesNo, MessageBoxIcon.Question);
            if (dr != DialogResult.Yes) return;
            #endregion

            //tb_1.Text = vPreviousState[0];
            //tb_2.Text = vPreviousState[1];
            //tb_3.Text = vPreviousState[2];
            //tb_4.Text = vPreviousState[3];
            //tb_Biodynamics.Text = vPreviousState[4];
            tb_FicheMed.Text = vPreviousState[5];
            //tb_Kinergetique.Text = vPreviousState[6];
            //tb_HE.Text = vPreviousState[7];
            UpdateCards();
            vIsSaved = false;
        }
        /// <summary>
        /// Menu Quitter
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Quit(object sender, EventArgs e)
        {
            this.Close();
        }
        /// <summary>
        /// Sauve et quitte Pata
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SaveNQuit_Click(object sender, EventArgs e)
        {
            SaveCards();
            Close();
        }
        /// <summary>
        /// Demande une modification du patient courrant
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_PatMod(object sender, EventArgs e)
        {
            if (tssl_Name.Text != "Session ferme" && vPatientManager.IDPatient > 0)//&& vPatientInfo.IsFilled)
            {
                fr_PatientManager fr = new fr_PatientManager();
                fr.Size = new Size(fr.Size.Width, 600);
                fr.Manager = vPatientManager;
                fr.Use = Use.Update;
                fr.Text = "Gestion de patient...";
                fr.ShowDialog();
                tssl_Name.Text = fr.Manager.ToString();

            }
        }
        /// <summary>
        /// Suppression du patient courrant
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelPatient(object sender, EventArgs e)
        {
            try
            {
                if (tssl_Name.Text != "Session ferme" && vPatientManager.IDPatient > 0)//&& vPatientInfo.IsFilled)
                {
                    DialogResult dr = MessageBox.Show("Voulez-vous rellement supprimer "
                        + tssl_Name.Text + "?",
                        "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                    PatientManager pat = new PatientManager();
                    pat.IDPatient = vPatientManager.IDPatient;

                    if (dr == DialogResult.Yes)
                    {
                        pat.Delete();
                        tssl_Name.Text = "Session ferme";
                        tsMenu_Supprimer.Enabled = false;
                        tsMenu_Modification.Enabled = false;
                    }
                }
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex.Message, this, LogType.Error);
                MessageBox.Show("Impossible de supprimer le patient.\r\n" + ex.Message,
                    "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }
        }
        /// <summary>
        /// Evnement sur une demande de sauvegarde
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Save(object sender, EventArgs e)
        {
            SaveCards();
        }
        /// <summary>
        /// Clique sur A propos
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_APropos(object sender, EventArgs e)
        {
            fr_About fr = new fr_About();
            fr.ShowDialog();
        }
        /// <summary>
        /// Click in delete prescription
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DeletePrescription(object sender, EventArgs e)
        {
            if (dgv_Prescription.SelectedRows.Count <= 0) return;
            int vID = (int)dgv_Prescription.SelectedRows[0].Cells["id_Prescription"].Value;
            PrescriptionManager pm = PrescriptionManager.Create(vID);
            fr_PrescriptionManager fr = new fr_PrescriptionManager();
            fr.Use = Use.Delete;
            fr.Manager = pm;
            fr.Size = new Size(fr.Size.Width, 250);
            fr.ShowDialog();
            this.FillWithPrescription();
        }
        /// <summary>
        /// Click on modify prescription
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModifyPrescription(object sender, EventArgs e)
        {
            if (dgv_Prescription.SelectedRows.Count <= 0) return;
            int vID = (int)dgv_Prescription.SelectedRows[0].Cells["id_Prescription"].Value;
            PrescriptionManager pm = PrescriptionManager.Create(vID);
            fr_PrescriptionManager fr = new fr_PrescriptionManager();
            fr.Use = Use.Update;
            fr.Manager = pm;
            fr.Size = new Size(fr.Size.Width, 250);
            fr.ShowDialog();
            this.FillWithPrescription();
        }
        /// <summary>
        /// Opening of the context menu
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ContextMenu_Opening(object sender, CancelEventArgs e)
        {
            if (dgv_Prescription.SelectedRows.Count != 1)
            {
                tsmi_ModifyPrescription.Enabled = false;
                tsmi_DelPrescription.Enabled = false;
            }
            else
            {
                tsmi_ModifyPrescription.Enabled = true;
                tsmi_DelPrescription.Enabled = true;
            }
        }
        /// <summary>
        /// Click in "Manage prescriptions"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ManagePrescriptions(object sender, EventArgs e)
        {
            fr_Prescriptions fr = new fr_Prescriptions();
            fr.PatientManager = this.vPatientManager;
            fr.ShowDialog();
        }
        /// <summary>
        /// Mouse Down on the DataGridView
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_MouseDown_DataGridView(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                DataGridView.HitTestInfo HitTestInfo = this.dgv_Prescription.HitTest(e.X, e.Y);
                if (HitTestInfo.RowIndex > -1 && dgv_Prescription.SelectedRows.Count > 0)
                {
                    this.dgv_Prescription.SelectedRows[0].Selected = false;
                    this.dgv_Prescription.Rows[HitTestInfo.RowIndex].Selected = true;
                }
            }
        }
        /// <summary>
        /// Evnement sur changement de texte
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_TextChanged(object sender, EventArgs e)
        {
            if (vIsLoading)
                vIsLoading = false;
            else
                vIsSaved = false;
        }
        /// <summary>
        /// Evnement sur fermeture de la fentre
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_FormClosing(object sender, FormClosingEventArgs e)
        {
            #region Location of the menus
            //Options.Instance.PataMenu.GeneralLocation = ts_General.Location;
            //Options.Instance.PataMenu.PageStyleLocation = ts_PageStyle.Location;
            Options.Instance.PataMenu.SeekCancelLocation = ts_Menu.Location;
            #endregion

            if (!vIsSaved && vPatientManager != null)
            {
                DialogResult dr = MessageBox.Show("Voulez-vous sauver les modifications ralises?",
                     "Attention", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Warning);

                if (dr == DialogResult.Yes)
                {
                    try { UpdateCards(); }
                    catch (NotSelectedException ex)
                    {
                        MessageBox.Show(ex.Message, "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
                else if (dr == DialogResult.Cancel)
                {
                    e.Cancel = true;
                }
            }
        }
        /// <summary>
        /// The frame is loading
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_FrameLoad(object sender, EventArgs e)
        {
            ts_Menu.Location = Options.Instance.PataMenu.SeekCancelLocation;
            this.UpdatePatientData();
        }
        /// <summary>
        /// Focus on the frame
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_FrameEnter(object sender, EventArgs e)
        {
            this.UpdatePatientData();
        }
        /// <summary>
        /// Evnement sur un changement d'vOptions
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_OptionChanged(object sender, EventArgs e)
        {
            SetTabs();
            SetToolStrips();
        }
        /// <summary>
        /// Event Prescription's tab selected
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_EnterTab_Prescription(object sender, EventArgs e)
        {
            if (this.vPatientManager != null && this.vPatientManager.IDPatient != -1)
            {
                FillWithPrescription();
            }
        }
        /// <summary>
        /// Click add prescription
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_AddPrescription(object sender, EventArgs e)
        {
            try
            {
                if (this.vPatientManager == null || this.vPatientManager.IDPatient < 0)
                {
                    MessageBox.Show("Veuillez selectionner un patient! (Menu de selection de patient)", "Attention",
                         MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
                else
                {
                    PhoenixPrescription.GUI.fr_Main fr = new PhoenixPrescription.GUI.fr_Main();
                    fr.SelectedPatient = this.vPatientManager;
                    fr.ShowDialog();
                    FillWithPrescription();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Impossible d'ajouter une prescription!", "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.Instance.Write(ex, this, LogType.Error);
            }
        }
        /// <summary>
        /// Clique sur la barre de status du patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_StatusName(object sender, EventArgs e)
        {
            if (tssl_Name.Text != "Session ferme" && vPatientManager.IDPatient > 0)//&& vPatientInfo.IsFilled)
            {
                fr_PatientManager fr = new fr_PatientManager();
                fr.Size = new Size(fr.Size.Width, 600);
                fr.Manager = vPatientManager;
                fr.Use = Use.Update;
                fr.Text = "Gestion de patient...";
                fr.ShowDialog();
                tssl_Name.Text = fr.Manager.ToString();
                tssl_Updated.Text = "Dernire mj " + fr.Manager.LastUpdate.ToShortDateString();
                UpdateAge();
                if (Mailer.IsValid(fr.Manager.Mail))
                    tsb_SendMail.Visible = true;
                else
                    tsb_SendMail.Visible = false;
                this.UpdatePatientData();
            }
        }
        /// <summary>
        /// Event on meetings management 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_MeetingManage(object sender, EventArgs e)
        {
            fr_HistoryNonCameMeeting fr = new fr_HistoryNonCameMeeting();
            fr.ActivatePatient(vTimeCardManager.PatientManager.IDPatient);

            if (sender == this.tssm_Uncame)
                fr.FrameUse = FrameUse.UncameMeetings;
            else if (sender == this.tssm_Unpayed)
                fr.FrameUse = FrameUse.UnpayedMeetings;

            fr.ShowDialog();
        }
        /// <summary>
        /// Click on "manage de debt"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_Debt(object sender, EventArgs e)
        {
            fr_DebtManager fr = new fr_DebtManager();
            fr.PatientManager = this.vTimeCardManager.PatientManager;
            DialogResult dr = fr.ShowDialog();
            if (dr == DialogResult.Cancel) return;            
            this.UpdatePatientData();
        }
        /// <summary>
        /// Click on a ToolStripSpliButton
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_ToolStripSplitButton(object sender, EventArgs e)
        {
            ToolStripSplitButton tssb = (ToolStripSplitButton)sender;
            tssb.ShowDropDown();
        }
        /// <summary>
        /// Send a mail to the patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_SendMail(object sender, EventArgs e)
        {
            if (this.vPatientManager.Empty) return;
            System.Diagnostics.Process.Start("mailto:" + this.vPatientManager.Mail);
        }

        #region Picture displaying
        /// <summary>
        /// The thumbs ListBox wants to be redrawed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        //TODO Check if useful
        private void Event_ListBoxDrawItem(object sender, DrawItemEventArgs e)
        {
            try
            {
                if (e.Index < 0 || il_ThumbList.Images.Count <= e.Index) return;
                Point p = e.Bounds.Location;
                il_ThumbList.Draw(e.Graphics, p, e.Index);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Une erreur innatendue est survenue! (" + ex.Message + ")",
                    "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.Instance.Write(ex, this, LogType.Error);
            }
        }
        /// <summary>
        /// Double click on the PictureBox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DoubleClick_PictureBox(object sender, EventArgs e)
        {
            if (pb_Picture.Image == null) return;

            fr_FullScreenPicture fr = new fr_FullScreenPicture();
            fr.Image = pb_Picture.Image;
            fr.ShowDialog();
        }
        /// <summary>
        /// Delete the selected pic
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_PicDel(object sender, EventArgs e)
        {
            try
            {
                DialogResult dr =
                    MessageBox.Show("Voulez-vous supprimer cette photo?", "Question", MessageBoxButtons.YesNo, MessageBoxIcon.Question);

                if (dr == DialogResult.Yes)
                {
                    SqlManager.ExecuteNonQuery("DELETE * FROM Photo WHERE id_Photo = " + (string)this.lv_Thumb.SelectedItems[0].Text);
                    pb_Picture.Image = null;
                    lv_Thumb.Items.RemoveAt(this.lv_Thumb.SelectedItems[0].Index);
                    //il_ThumbList.Images.RemoveAt(this.lv_Thumb.SelectedItems[0].Index);
                    btn_PicModify.Enabled = false;
                    btn_PicDel.Enabled = false;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Impossible de supprimer la photo!", "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.Instance.Write(ex, this, LogType.Error);
            }
        }
        /// <summary>
        /// Modify the selected pic
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>        
        private void Click_PicModify(object sender, EventArgs e)
        {
            if (btn_PicModify.Text.ToLower() == "Activer Modifier".ToLower())
            {
                tb_Remarks.ReadOnly = false;
                btn_PicModify.Text = "Modifier";
            }
            else
                if (lv_Thumb.SelectedItems[0] != null)
                {
                    SqlManager.ExecuteNonQuery("UPDATE Photo SET Remarques = \""
                            + tb_Remarks.Text + "\" WHERE id_Photo = " + this.lv_Thumb.SelectedItems[0].Text);
                    tb_Remarks.ReadOnly = true;
                    btn_PicModify.Enabled = false;
                    btn_PicModify.Text = "Activer Modifier";
                }
        }
        /// <summary>
        /// Add a pic to the patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_PicAdd(object sender, EventArgs e)
        {
            if (vPatientManager.IDPatient == -1) return;

            DialogResult dr = ofd_PicturesSelection.ShowDialog();

            if (dr == DialogResult.OK)
            {
                tspb_ProgressBar.Visible = true;
                Image bmp;
                MemoryStream ms;
                SqlManager.Open();
                tspb_ProgressBar.Maximum = ofd_PicturesSelection.FileNames.Length * 5;
                tspb_ProgressBar.Minimum = 0;
                tspb_ProgressBar.Step = 1;
                il_ThumbList = lv_Thumb.LargeImageList;
                try
                {
                    foreach (string file in ofd_PicturesSelection.FileNames)
                    {
                        ms = new MemoryStream();
                        SqlManager.PrepareProcedure("INSERT_Photo", false);
                        SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = vPatientManager.IDPatient;
                        tspb_ProgressBar.PerformStep();

                        bmp = new Bitmap(file);
                        bmp.Save(ms, ImageFormat.Jpeg);
                        SqlManager.Parameters.Add("[_Pic]", OleDbType.Binary).Value = ms.ToArray();
                        tspb_ProgressBar.PerformStep();

                        bmp = ImageResizer.Resize((Bitmap)bmp, 10);
                        bmp.Save(ms, ImageFormat.Jpeg);
                        SqlManager.Parameters.Add("[_Thumb]", OleDbType.Binary).Value = ms.ToArray();
                        tspb_ProgressBar.PerformStep();

                        string rmq = "";
                        if (ofd_PicturesSelection.FileNames.Length <= 1)
                        {
                            fr_Remarks fr = new fr_Remarks();
                            fr.Image = bmp;
                            DialogResult drr = fr.ShowDialog();
                            rmq = fr.Remark;
                        }

                        SqlManager.Parameters.Add("[_Rmq]", OleDbType.Variant).Value = rmq;
                        tspb_ProgressBar.PerformStep();

                        object LastId = 0;
                        SqlManager.ExecuteNonQueryProcedure(ref LastId);
                        SqlManager.Parameters.Clear();
                        il_ThumbList.Images.Add(bmp);

                        lv_Thumb.Items.Add(LastId.ToString()).ImageIndex = il_ThumbList.Images.Count - 1; ;
                        tspb_ProgressBar.PerformStep();
                    }

                }
                catch (Exception ex)
                {
                    Log.Instance.Write(ex.Message, this, LogType.Error);
                    MessageBox.Show("Une erreur est survenue lors de l'ajout d'une photo.\r\n" + ex.Message,
                        "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                lv_Thumb.LargeImageList = il_ThumbList;
                SqlManager.Close();
                tspb_ProgressBar.Value = 0;
                tspb_ProgressBar.Visible = false;
            }
        }
        /// <summary>
        /// The thumb selection changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_ThumbSelectedItemChanged(object sender, EventArgs e)
        {
            try
            {
                if (lv_Thumb.SelectedItems.Count <= 0) return;
                SqlManager.SqlQuery = "SELECT * FROM photo WHERE id_photo = " + lv_Thumb.SelectedItems[0].Text;
                DataSet ds = new DataSet();
                SqlManager.ExecuteSql(ref ds);

                pb_Picture.Image = new Bitmap(new MemoryStream((byte[])ds.Tables[0].Rows[0]["BinPic"]));
                tb_Remarks.Text = (string)ds.Tables[0].Rows[0]["Remarques"];
                btn_PicDel.Enabled = true;
                btn_PicModify.Enabled = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Une erreur est survenue!", "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.Instance.Write(ex, this, LogType.Error);
            }
        }
        #endregion

        #region Mise en page
        /// <summary>
        /// Ajout la date dans le fentre courrante
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddDate(object sender, EventArgs e)
        {
            string AddDate = "\r\n" + DateTime.Today.ToShortDateString()
                + "\r\n==============================\r\n\r\n";
            AddInTab(AddDate);
        }
        /// <summary>
        /// Ajoute un sparateur dans la entre courrante
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddSeparator(object sender, EventArgs e)
        {
            string Separator = "\r\n=====================================" +
                "===========================================\r\n\r\n";
            AddInTab(Separator);
        }
        /// <summary>
        /// Add the patient's age in the current frame
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddAge(object sender, EventArgs e)
        {
            if (vPatientManager == null) return;
            string Msg = "\r\n";
            #region Age
            // get the difference in years
            int years = DateTime.Now.Year - vPatientManager.DateOfBirth.Year;
            // subtract another year if we're before the
            // birth day in the current year
            if (DateTime.Now.Month < vPatientManager.DateOfBirth.Month
                || (DateTime.Now.Month == vPatientManager.DateOfBirth.Month
                && DateTime.Now.Day < vPatientManager.DateOfBirth.Day))
                years--;
            Msg += "Age: " + years + " ans";
            DateTime startDate = DateTime.Now;
            DateTime endDate = vPatientManager.DateOfBirth;
            int monthsApart = 12 - (Math.Abs(startDate.Month - endDate.Month));
            monthsApart = Math.Abs(monthsApart);
            Msg += " " + monthsApart + " mois";
            #endregion
            AddInTab(Msg);
        }
        /// <summary>
        /// Adds text in one of the TextBoxes
        /// </summary>
        /// <param name="_TextToAppend">Text to insert</param>
        private void AddInTab(string _TextToAppend)
        {
            if (vPatientManager == null) return;
            if (tc_General.SelectedTab == tab_FicheMed)
            {
                Clipboard.SetDataObject(_TextToAppend);
                this.tb_FicheMed.SelectionLength = 0;
                this.tb_FicheMed.SelectionStart -= this.tb_FicheMed.SelectionLength;
                this.tb_FicheMed.Paste();
                //tb_Kinergetique.Text += _TextToAppend;
            }
        }
        #endregion

        #region Menu Gestion
        /// <summary>
        /// Clique sur une demande de menu de gestion
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SeekCancel(object sender, EventArgs e)
        {
            if (!tsMenu_SeekCancel.Checked)
            {
                tsMenu_SeekCancel.Checked = true;
                ts_Menu.Visible = true;
            }
            else
            {
                tsMenu_SeekCancel.Checked = false;
                ts_Menu.Visible = false;
            }
            SetIconMenus();
        }
        /// <summary>
        /// Click on incomplete patients management
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ManagePatients(object sender, EventArgs e)
        {
            fr_ManageIncomplete fr = new fr_ManageIncomplete();
            fr.ShowDialog();
        }

        #region Ajout
        /// <summary>
        /// Menu d'ajout de rputation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddReput(object sender, EventArgs e)
        {
            fr_ReputationManager fr = new fr_ReputationManager();
            fr.Manager = new ReputationManager();
            fr.Text = "Gestion des rputations...";
            fr.Use = Use.Add;
            fr.ShowDialog();
        }
        /// <summary>
        /// Ajout d'un honoraire
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddMutuelle(object sender, EventArgs e)
        {
            fr_MutuelleManager fr = new fr_MutuelleManager();
            fr.Manager = new MutuelleManager();
            fr.Text = "Gestion des mutuelles...";
            fr.Use = Use.Add;
            fr.ShowDialog();
        }
        /// <summary>
        /// Ajoute un mdecin
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddDoctor(object sender, EventArgs e)
        {
            fr_DoctorManager fr = new fr_DoctorManager();
            fr.Manager = new DoctorManager();
            fr.Text = "Gestion des mdecins...";
            fr.Use = Use.Add;
            fr.Size = new Size(327, 381);
            fr.ShowDialog();
        }
        /// <summary>
        /// Ajout d'un mdicament
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddDrug(object sender, EventArgs e)
        {
            fr_DrugsManager fr = new fr_DrugsManager();
            fr.Manager = new DrugsManager();
            fr.Text = "Gestion des mdicaments";
            fr.Use = Use.Add;
            fr.ShowDialog();
        }
        /// <summary>
        /// Ajout d'un type de mdicament        
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddDrugType(object sender, EventArgs e)
        {
            fr_DrugsTypeManager fr = new fr_DrugsTypeManager();
            fr.Manager = new DrugsTypeManager();
            fr.Use = Use.Add;
            fr.ShowDialog();
        }
        /// <summary>
        /// Evnement sur une demande d'ajotu de patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AddPatient(object sender, EventArgs e)
        {
            fr_PatientManager fr = new fr_PatientManager();
            fr.Use = Use.Add;
            fr.Size = new Size(fr.Size.Width, 600);
            fr.Text = "Gestion de patient...";
            fr.ShowDialog();

            if (fr.DialogResult != DialogResult.OK) return;
            fr.Manager.DateInscription = DateTime.Today;
            fr.Manager.LastUpdate = DateTime.Today;
            SetFrameOnPatientSelection(fr.Manager, new CardsInfo());
        }
        #endregion

        #region Modification
        /// <summary>
        /// Modification d'un honoraire
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModMutuelle(object sender, EventArgs e)
        {
            fr_MutuelleSeeker fr = new fr_MutuelleSeeker();
            fr.Text = "Recherche d'une mutuelle...";
            fr.btn_AddReputation.Visible = false;
            DialogResult dr = fr.ShowDialog();

            if (dr == DialogResult.OK)
            {
                fr_MutuelleManager hm = new fr_MutuelleManager();
                hm.Manager = fr.SelectedItem;
                hm.Text = "Modification d'une mutuelle...";
                hm.Use = Use.Update;
                hm.ShowDialog();
            }
        }
        /// <summary>
        /// Modification d'une rputation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModReput(object sender, EventArgs e)
        {
            fr_ReputationSeeker fr = new fr_ReputationSeeker();
            fr.Text = "Recherche d'une rputation";
            fr.btn_AddReputation.Visible = false;
            DialogResult dr = fr.ShowDialog();

            if (dr == DialogResult.OK)
            {
                fr_ReputationManager rm = new fr_ReputationManager();
                rm.Manager = fr.SelectedItem;
                rm.Text = "Modification d'une rputation...";
                rm.Use = Use.Update;
                rm.ShowDialog();
            }
        }
        /// <summary>
        /// Modification d'un mdecin
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModDoctor(object sender, EventArgs e)
        {
            fr_DoctorSeeker fr = new fr_DoctorSeeker();
            fr.Text = "Recherche d'un mdecin...";
            fr.btn_NoneDoctor.Visible = false;
            DialogResult dr = fr.ShowDialog();

            if (dr == DialogResult.OK)
            {
                fr_DoctorManager dm = new fr_DoctorManager();
                dm.Manager = fr.SelectedItem;
                dm.Text = "Modification d'un mdecin...";
                dm.Use = Use.Update;
                dm.Size = new Size(327, 381);

                dm.ShowDialog();
            }

        }
        /// <summary>
        /// Modification of a drug
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModDrug(object sender, EventArgs e)
        {
            fr_DrugsSeeker fr = new fr_DrugsSeeker();
            fr.Text = "Recherche de mdicament...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_DrugsManager dm = new fr_DrugsManager();
            dm.Manager = fr.SelectedItem;
            dm.Text = "Modification d'un mdicament";
            dm.Use = Use.Update;
            dm.Size = new Size(327, 350);
            dm.ShowDialog();
        }
        /// <summary>
        /// Modification of a drug type
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModDrugType(object sender, EventArgs e)
        {
            fr_DrugsTypeSeeker fr = new fr_DrugsTypeSeeker();
            fr.Text = "Recherche d'un type de mdicament...";
            fr.btn_AddReputation.Visible = false;
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_DrugsTypeManager dtm = new fr_DrugsTypeManager();
            dtm.Manager = fr.SelectedItem;
            dtm.Text = "Modification d'un type de mdicament";
            dtm.Use = Use.Update;
            dtm.Size = new Size(327, 350);
            dtm.ShowDialog();
        }
        /// <summary>
        /// Moodification of a A patient
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ModAPatient(object sender, EventArgs e)
        {
            fr_PatientSeeker fr = new fr_PatientSeeker();
            fr.Text = "Recherche d'un patient...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_PatientManager pm = new fr_PatientManager();
            pm.Manager = fr.SelectedItem;
            pm.Text = "Modification d'un patient...";
            pm.Use = Use.Update;
            pm.Size = new Size(pm.Size.Width, 600);
            pm.ShowDialog();

        }
        #endregion

        #region Suppression
        /// <summary>
        /// Suppression de mdecin
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelDoc(object sender, EventArgs e)
        {
            fr_DoctorSeeker fr = new fr_DoctorSeeker();
            fr.Text = "Recherche d'un mdecin...";
            fr.btn_NoneDoctor.Visible = false;
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_DoctorManager dm = new fr_DoctorManager();
            dm.Text = "Suppression de ce mdecin";
            dm.Manager = fr.SelectedItem;
            dm.Use = Use.Delete;
            dm.Size = new Size(dm.Size.Width, 400);
            dm.ShowDialog();
        }
        /// <summary>
        /// Suppression d'une rputation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelReput(object sender, EventArgs e)
        {
            fr_ReputationSeeker fr = new fr_ReputationSeeker();
            fr.Text = "Recherche d'une rputation...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_ReputationManager rm = new fr_ReputationManager();
            rm.Text = "Suppression de cette rputation";
            rm.Manager = fr.SelectedItem;
            rm.Use = Use.Delete;
            rm.Size = new Size(rm.Size.Width, 250);
            rm.ShowDialog();
        }
        /// <summary>
        /// Suppression d'un mdicament
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelDrug(object sender, EventArgs e)
        {
            fr_DrugsSeeker fr = new fr_DrugsSeeker();
            fr.Text = "Recherche d'un mdicament...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_DrugsManager dm = new fr_DrugsManager();
            dm.Text = "Suppression de ce mdicament";
            dm.Manager = fr.SelectedItem;
            dm.Use = Use.Delete;
            dm.Size = new Size(dm.Size.Width, 250);
            dm.ShowDialog();
        }
        /// <summary>
        /// Suppression d'une mutuelle
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelMutuelle(object sender, EventArgs e)
        {
            fr_MutuelleSeeker fr = new fr_MutuelleSeeker();
            fr.Text = "Recherche d'une mutuelle";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_MutuelleManager mm = new fr_MutuelleManager();
            mm.Text = "Suppression de cette mutuelle";
            mm.Manager = fr.SelectedItem;
            mm.Use = Use.Delete;
            mm.Size = new Size(mm.Size.Width, 250);
            mm.ShowDialog();
        }
        /// <summary>
        /// Suppression d'un type de mdicament
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DelDrugType(object sender, EventArgs e)
        {
            fr_DrugsTypeSeeker fr = new fr_DrugsTypeSeeker();
            fr.Text = "Recherche d'un type de mdicament...";
            DialogResult dr = fr.ShowDialog();

            if (dr != DialogResult.OK) return;
            fr_DrugsTypeManager dtm = new fr_DrugsTypeManager();
            dtm.Text = "Suppression de ce type de mdicament";
            dtm.Manager = fr.SelectedItem;
            dtm.Use = Use.Delete;
            dtm.Size = new Size(dtm.Size.Width, 250);
            dtm.ShowDialog();
        }
        #endregion

        #endregion
        #endregion

        #region Methods
        /// <summary>
        /// Mise  jour des fiches mdicales
        /// </summary>
        protected bool UpdateCards()
        {
            try
            {
                if (vPatientManager.IDPatient > 0)//(vPatientInfo.IsFilled)
                {
                    #region OldCode
                    #endregion
                    //TODO : Check query
                    SqlManager.PrepareProcedure("UPDATE_FicheMedicale");
                    SqlManager.Parameters.Add("_ID", OleDbType.Integer).Value = vPatientManager.IDPatient;
                    SqlManager.Parameters.Add("_FicheMedicale", OleDbType.Variant).Value = tb_FicheMed.Text;
                    #region OBSOLETE
                    SqlManager.Parameters.Add("_Biodynamics", OleDbType.Variant).Value = "";
                    SqlManager.Parameters.Add("_HE", OleDbType.Variant).Value = "";
                    SqlManager.Parameters.Add("_Kinergetique", OleDbType.Variant).Value = "";
                    SqlManager.Parameters.Add("_FICHE_SUPPL_1", OleDbType.Variant).Value = "";
                    SqlManager.Parameters.Add("_FICHE_SUPPL_2", OleDbType.Variant).Value = "";
                    SqlManager.Parameters.Add("_FICHE_SUPPL_3", OleDbType.Variant).Value = "";
                    SqlManager.Parameters.Add("_FICHE_SUPPL_4", OleDbType.Variant).Value = "";
                    #endregion
                    SqlManager.Parameters.Add("_MAJ", OleDbType.Date).Value = DateTime.Now;
                    SqlManager.ExecuteNonQueryProcedure();
                    return true;
                }
                else
                    throw new NotSelectedException("Aucun patient de selectionn");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Une erreur est survenue lors de la mise  jour des fiches mdicales!",
                    "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.Instance.Write(ex, this, LogType.Error);
                return false;
            }
        }
        /// <summary>
        /// Initialise les onglets  afficher
        /// </summary>
        protected void SetTabs()
        {
            /*try
            {
                tc_General.TabPages.Clear();

                foreach (TabPage tp in vTabPage)
                {
                    tc_General.TabPages.Add(tp);
                }

                if (!opt.IsVisible1)
                    tc_General.TabPages.RemoveByKey("tab_1");
                else
                    tc_General.TabPages["tab_1"].Text = opt.AdditionnalTitle1;

                if (!opt.IsVisible2)
                    tc_General.TabPages.RemoveByKey("tab_2");
                else
                    tc_General.TabPages["tab_2"].Text = opt.AdditionnalTitle2;

                if (!opt.IsVisible3)
                    tc_General.TabPages.RemoveByKey("tab_3");
                else
                    tc_General.TabPages["tab_3"].Text = opt.AdditionnalTitle3;

                if (!opt.IsVisible4)
                    tc_General.TabPages.RemoveByKey("tab_4");
                else
                    tc_General.TabPages["tab_4"].Text = opt.AdditionnalTitle4;
            }
            catch (ObjectDisposedException ex)
            { Log.Instance.Write(ex.Message, "PhoenixCars.fr_Main.SetTabs()", LogType.Error); }*/
        }
        /// <summary>
        /// Initialise les valeurs pour les menu  icones
        /// </summary>
        protected void SetIconMenus()
        {
            PataMenu pm = Options.Instance.PataMenu;
            pm.SeekCancelVisible = tsMenu_SeekCancel.Checked;
            Options.Instance.PataMenu = pm;
        }
        /// <summary>
        /// Saves the cards
        /// </summary>
        private void SaveCards()
        {
            try
            {
                if (vPatientManager != null)
                {
                    if (UpdateCards())
                    {
                        vIsSaved = true;
                        tssl_Updated.Text = "Dernire maj: " + DateTime.Now.ToShortDateString();
                        MessageBox.Show("Enregistrement effectu", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                }
                else
                    MessageBox.Show("Veuillez selectionnner un patient avant d'enregistrer!", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (NotSelectedException ex)
            {
                MessageBox.Show(ex.Message, "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        /// <summary>
        /// Sets the size of the image of the menus
        /// </summary>
        private void SetToolStrips()
        {
            ts_Menu.ImageScalingSize = Options.Instance.MenuImageScalingSize;
            tss_Sep1.Size = Options.Instance.MenuImageScalingSize;
            tss_Sep2.Size = Options.Instance.MenuImageScalingSize;
            ts_Menu.AutoSize = true;
        }
        /// <summary>
        /// Fill the DataGridView with the prescriptions
        /// </summary>
        private void FillWithPrescription()
        {
            try
            {
                OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
                SqlManager.PrepareProcedure("SELECT_PrescriptionOnPatient");
                SqlManager.Parameters.Add("[_ID]", OleDbType.Integer).Value = vPatientManager.IDPatient;
                DataSet ds = new DataSet();
                SqlManager.ExecuteSqlProcedure(ref ds);
                dgv_Prescription.DataSource = ds.Tables[0];

                for (int i = 0; i < dgv_Prescription.Columns.Count; i++)
                    dgv_Prescription.Columns[i].Visible = false;

                dgv_Prescription.Columns["Nom"].Visible = true;
                dgv_Prescription.Columns["Nom"].HeaderText = "Mdicament";
                dgv_Prescription.Columns["Nom"].DisplayIndex = 0;

                dgv_Prescription.Columns["p.Rmq"].Visible = true;
                dgv_Prescription.Columns["p.Rmq"].HeaderText = "Remarques";
                dgv_Prescription.Columns["p.Rmq"].DisplayIndex = 1;

                dgv_Prescription.Columns["Date_P"].Visible = true;
                dgv_Prescription.Columns["Date_P"].HeaderText = "Date de prescription";
                dgv_Prescription.Columns["Date_P"].DisplayIndex = 2;
                dgv_Prescription.Sort(dgv_Prescription.Columns["Date_P"], ListSortDirection.Ascending);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Exclamation", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
        }
        /// <summary>
        /// Updates the information of the patient
        /// </summary>
        private void UpdatePatientData()
        {
            #region Today
            try
            {
                if (this.vPatientManager == null)
                {
                    tssl_HasRdv.Text = "";
                    tssl_HasRdv.BackColor = SystemColors.Control;
                    return;
                }
                //Check meetings of today
                this.vPatientManager = PatientManager.Create(vPatientManager.IDPatient);
                vTimeCardManager = new TimeCardManager(this.vPatientManager);
                if (vTimeCardManager.HasRendezVous)
                {
                    tssl_HasRdv.Text = "Rendez-vous aujourd'hui!";
                    tssl_HasRdv.BackColor = Color.Red;
                    tsb_Today.Visible = true;
                    tsMenu_Today.Visible = true;
                    tss_Separator.Visible = true;
                }
                else
                {
                    tssl_HasRdv.Text = "";
                    tssl_HasRdv.BackColor = SystemColors.Control;
                    tsb_Today.Visible = false;
                    tsMenu_Today.Visible = false;

                    tss_Separator.Visible = false;
                }
                if (vTimeCardManager.PatientManager.Debt == 0)
                {
                    tsMenu_Debt.Visible = false;
                    tsb_Debt.Visible = false;
                    tss_Separator.Visible = false;
                }
                else
                {
                    tsb_Debt.Visible = true;
                    tsMenu_Debt.Visible = true;
                    tss_Separator.Visible = true;
                }
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
            }
            #endregion

            #region Debt
            tssl_Debt.Text = "Dette : " + vTimeCardManager.PatientManager.Debt + " " + Options.Instance.MoneySign;
            if (vTimeCardManager.PatientManager.Debt == 0)
                tssl_Debt.Text = "";
            #endregion

            #region Missed Meetings
            int rows = vTimeCardManager.MissedMeetings;
            if (rows <= 0)
                tssddb_MissedMeeting.Visible = false;
            else
            {
                tssddb_MissedMeeting.Visible = true;
                tssddb_MissedMeeting.Text = rows + " rendez-vous manqu(s)";
            }
            #endregion

            #region Check pictures
            if (!vIsLoading && tlp_GeneralPics.Location.X != 9999)
            {
                SqlManager.SqlQuery = "SELECT COUNT(*) FROM photo WHERE id_patient = " + vPatientManager.IDPatient + ";";
                int count = (int)SqlManager.ExecuteScalar();
                if (count != lv_Thumb.Items.Count)
                {
                    DialogResult dr = MessageBox.Show("Il y a eu modification des photos du patient. Voulez-vous recharger les photos?",
                        "Attention!", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk);
                    if (dr == DialogResult.No)
                    {
                        lbl_WaitMessage.Dock = DockStyle.None;
                        tlp_GeneralPics.Dock = DockStyle.None;
                        lbl_WaitMessage.Location = new Point(0, 0);
                        tlp_GeneralPics.Location = new Point(9999, 9999);
                        lbl_WaitMessage.Dock = DockStyle.Fill;
                        lbl_WaitMessage.Text = "Les photos de ce patient on t modifies ailleur!";
                    }
                    if (dr == DialogResult.Yes)
                    {
                        RefreshPics();
                    }
                }
            }
            #endregion
        }
        /// <summary>
        /// Set the frame after a patient selection
        /// </summary>
        /// <param name="_SelectedItem"></param>
        /// <param name="_CardsInfo"></param>
        private void SetFrameOnPatientSelection(PatientManager _SelectedItem, CardsInfo _CardsInfo)
        {
            tssl_Name.Text = _SelectedItem.Surname + " " + _SelectedItem.FamillyName;
            tssl_Updated.Text = "Dernire mj: " + _SelectedItem.LastUpdate.ToShortDateString();

            tsMenu_Modification.Enabled = true;
            tsMenu_Modification.Text = "&Modification de " + tssl_Name.Text;
            tsMenu_Supprimer.Enabled = true;
            tsMenu_Supprimer.Text = "&Suppression de " + tssl_Name.Text;
            tsMenu_Previous.Enabled = true;
            tsb_Cancel.Enabled = true;
            btn_AddPrescription.Enabled = true;
            tsm_ManagePresc.Enabled = true;
            tsb_ManagePresc.Visible = true;
            tsb_AddAge.Visible = true;
            tsb_Separator.Visible = true;
            tsb_Date.Visible = true;

            if (Mailer.IsValid(_SelectedItem.Mail))
                tsb_SendMail.Visible = true;
            else
                tsb_SendMail.Visible = false;

            #region Fill the cards
            vPatientManager = _SelectedItem;
            CardsInfo c = _CardsInfo;
            tb_FicheMed.Text = c.Cards;
            vPreviousState[5] = c.Cards;

            UpdateAge();
            #endregion

            #region Indicates the picture thread is seeking
            pb_Picture.Image = null;
            il_ThumbList.Images.Clear();
            btn_PicDel.Enabled = false;
            btn_PicModify.Enabled = false;
            tlp_GeneralPics.Dock = DockStyle.None;
            tlp_GeneralPics.Location = new Point(9999, 9999);
            lbl_WaitMessage.Location = new Point(0, 0);
            lbl_WaitMessage.Dock = DockStyle.Fill;
            lbl_WaitMessage.Text = "Recherche en cours...";
            #endregion

            #region Check meetings and fill prescriptions
            this.UpdatePatientData();
            this.FillWithPrescription();
            #endregion

            #region Select the thumbnails
            RefreshPics();
            #endregion
        }
        /// <summary>
        /// Update the display of the age of the patient
        /// </summary>
        private void UpdateAge()
        {
            #region Age
            // get the difference in years
            int years = DateTime.Now.Year - vPatientManager.DateOfBirth.Year;
            // subtract another year if we're before the
            // birth day in the current year
            if (DateTime.Now.Month < vPatientManager.DateOfBirth.Month
                || (DateTime.Now.Month == vPatientManager.DateOfBirth.Month
                && DateTime.Now.Day < vPatientManager.DateOfBirth.Day))
                years--;
            if (years > 0)
                tssl_Age.Text = "Age: " + years + " ans";
            else
            {
                DateTime startDate = DateTime.Now;
                DateTime endDate = vPatientManager.DateOfBirth;
                int monthsApart = 12 * (startDate.Year - endDate.Year) + startDate.Month - endDate.Month;
                monthsApart = Math.Abs(monthsApart);
                tssl_Age.Text = "Age: " + monthsApart + " mois";
            }
            #endregion
        }
        /// <summary>
        /// Refresh the pics
        /// </summary>
        private void RefreshPics()
        {
            pb_Picture.Image = null;
            btn_PicDel.Enabled = false;
            lv_Thumb.Clear();
            il_ThumbList.Images.Clear();
            SeekPics sp = new SeekPics();
            sp.Patient = vPatientManager;
            sp.Frame = this;
            Thread th = new Thread(new ThreadStart(sp.Process));
            th.Name = "Picture thread";
            th.Start();
        }
        #endregion

        #region Debugging
        /// <summary>
        /// Clique sur une recherche de mdecin
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_DoctorSeeking(object sender, EventArgs e)
        {
            fr_DoctorSeeker fr = new fr_DoctorSeeker();
            fr.ShowDialog();
        }
        /// <summary>
        /// Clique sur recherche de rputation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Reputation(object sender, EventArgs e)
        {
            fr_ReputationSeeker fr = new fr_ReputationSeeker();
            DialogResult dr = fr.ShowDialog();
            if (dr == DialogResult.OK)
                MessageBox.Show(fr.SelectedItem.ToString());
        }
        /// <summary>
        /// Clique sur une demande de listing d'honoraire
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Honoraire(object sender, EventArgs e)
        {
            fr_MutuelleSeeker fr = new fr_MutuelleSeeker();
            DialogResult dr = fr.ShowDialog();
            if (dr == DialogResult.OK)
                MessageBox.Show(fr.SelectedItem.ToString());
        }
        /// <summary>
        /// Menu pour voir les photos
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Photos(object sender, EventArgs e)
        {
            MessageBox.Show("Non, il n'y a plus de menu d'ajout de photo dans le mode DEBUG!",
                "Debug Folie", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
        }
        /// <summary>
        /// Viewer de photo
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_ViewPicture(object sender, EventArgs e)
        {
            MessageBox.Show("Non, il n'y a plus de menu d'ajout de photo dans le mode DEBUG!",
                "Debug Folie", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
        }
        /// <summary>
        /// Affiche le nouveau fr_PatientSeeker
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_NewPatientSeeker(object sender, EventArgs e)
        {
            fr_PatientSeeker fr = new fr_PatientSeeker();
            fr.ShowDialog();
        }
        /// <summary>
        /// Lancement du nouveau DoctorSeeker
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_NewDoctorSeeker(object sender, EventArgs e)
        {
            fr_DoctorSeeker fr = new fr_DoctorSeeker();
            fr.ShowDialog();
        }
        /// <summary>
        /// Nouveau selectionneur de type de mdicaments
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_NewTypeDrugSeeker(object sender, EventArgs e)
        {
            fr_DrugsTypeSeeker fr = new fr_DrugsTypeSeeker();
            fr.ShowDialog();

        }
        /// <summary>
        /// Nouveau selectionneur de mutuelle
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_NewMutuelle(object sender, EventArgs e)
        {
            fr_MutuelleSeeker fr = new fr_MutuelleSeeker();
            fr.ShowDialog();
        }
        /// <summary>
        /// Nouveau selectionneur de rputation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_NewReputation(object sender, EventArgs e)
        {
            fr_ReputationSeeker fr = new fr_ReputationSeeker();
            fr.ShowDialog();
        }
        /// <summary>
        /// Nouveau menu de recherche de Mdicament
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_SeekDrugs(object sender, EventArgs e)
        {
            fr_DrugsSeeker fr = new fr_DrugsSeeker();
            fr.ShowDialog();
        }
        /// <summary>
        /// LongTextEditor
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_LongTextEditor_Click(object sender, EventArgs e)
        {
            Components.TypeEditors.GUI.fr_LongTextEditor fr = new Components.TypeEditors.GUI.fr_LongTextEditor();

            DialogResult dr = fr.ShowDialog();
            if (dr != DialogResult.OK) return;

            MessageBox.Show("Txt: " + fr.SelectedText + "\r\nFont: " + fr.SelectedFont.ToString());

        }
        #endregion

        #region Properties
        /// <summary>
        /// Retourne une valeur booleenne qui indique si on a modifier le contenu des fiches
        /// </summary>
        public bool Saved
        {
            get
            {
                if (this.vIsLoading) return true;
                else return this.vIsSaved;
            }
        }
        /// <summary>
        /// Gets or sets the meetings to display in the Calendar
        /// </summary>
        public MeetingSet MeetingSet { get { return vMeetingSet; } set { vMeetingSet = value; } }
        #endregion

        
    }
    /// <summary>
    /// Thread class the the seek of the pictures
    /// </summary>
    internal class SeekPics
    {
        #region Variables
        /// <summary>
        /// Contains the pics of the patient
        /// </summary>
        private DataSet vDSPics = new DataSet();
        /// <summary>
        /// Indicates the ID of the selected patient
        /// </summary>
        private PatientManager vPatient;
        /// <summary>
        /// Reference on the main frame
        /// </summary>
        private fr_Main vFrame;

        /// <summary>
        /// Delegate for the GUI treatment
        /// </summary>        
        private delegate void UpdateGuiHandler();
        /// <summary>
        /// Mthode de tratement inter threads
        /// </summary>
        private UpdateGuiHandler UpdateGui;
        /// <summary>
        /// Sql manager
        /// </summary>
        private OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
        /// <summary>
        /// Contains the thumbs of the pictures
        /// </summary>
        private ImageList vImageList = new ImageList();
        #endregion

        #region Constructors
        /// <summary>
        /// Contructor
        /// </summary>
        public SeekPics() { }
        #endregion

        #region Methods
        /// <summary>
        /// Thread treatment
        /// </summary>
        public void Process()
        {
            try
            {
                //Log.Instance.RestartWatch();
                #region Seek pictures
                if (vPatient == null) return;
                vDSPics = new DataSet();
                SqlManager.SqlQuery = "SELECT id_photo, thumb FROM photo WHERE id_patient = " + vPatient.IDPatient + ";";
                SqlManager.ExecuteSql(ref vDSPics);
                #endregion

                #region Store the thumbs
                vImageList = new ImageList();
                vImageList.ImageSize = new Size(100, 100);
                vImageList.ColorDepth = ColorDepth.Depth24Bit;
                MemoryStream ms;
                Bitmap bmp;

                foreach (DataRow dr in vDSPics.Tables[0].Rows)
                {
                    ms = new MemoryStream((byte[])dr["thumb"]);
                    bmp = (Bitmap)Bitmap.FromStream(ms, true, false);
                    //vImageList.Images.Add(bmp);
                    vFrame.lv_Thumb.LargeImageList.Images.Add(bmp);
                }
                #endregion

                #region Display pics
                UpdateGui = new UpdateGuiHandler(this.RefreshGui);
                vFrame.Invoke(UpdateGui);
                #endregion
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
            }
            finally
            {
                vDSPics.Dispose();
                //Log.Instance.StopWatch("Display " + vDSPics.Tables[0].Rows.Count + " pics");
            }
        }
        /// <summary>
        /// Refreshes the GUI with the pictures
        /// </summary>
        private void RefreshGui()
        {
            try
            {
                for (int i = 0; i < vDSPics.Tables[0].Rows.Count; i++)
                {
                    vFrame.lv_Thumb.Items.Add(((int)vDSPics.Tables[0].Rows[i]["id_photo"]).ToString());
                    vFrame.lv_Thumb.Items[i].ImageIndex = i;
                }

                vFrame.lbl_WaitMessage.Location = new Point(9999, 9999);
                vFrame.lbl_WaitMessage.Dock = DockStyle.None;
                vFrame.tlp_GeneralPics.Dock = DockStyle.Fill;
                vFrame.tlp_GeneralPics.Location = new Point(0, 0);
                vFrame.lbl_WaitMessage.Text = "Pictures found!";
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                vFrame.lbl_WaitMessage.Text = "Impossible d'afficher les photos!";
            }
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the ID of the selected patient
        /// </summary>
        public PatientManager Patient
        {
            get { return vPatient; }
            set { vPatient = value; }
        }
        /// <summary>
        /// Gets or sets the main frame for the delegate treatment
        /// </summary>
        public fr_Main Frame
        {
            get { return vFrame; }
            set { vFrame = value; }
        }
        #endregion
    }
}